﻿from JGCalendar import *
import sys
import math
#■グレゴリオ暦からユリウス日に変換
def WorldToJDL(T):
    return  WorldToJD(T[0],T[1],T[2],T[3],T[4],T[5])

def WorldToJD(year,month,day,hour,minut,sec):
    if month < 3.0: year -= 1; month += 12
    jd  = int(365.25*year)+int(year/400.0)-int(year/100.0)+int(30.59*(month-2.0))
    jd += 1721088+day+(sec/3600.0+minut/60.0+hour)/24.0
    return jd
#■ユリウス日(JD:実数形式)から年月日時分秒(グレゴリオ暦)を計算する
#   戻り値　[年, 月, 日, 時, 分, 秒]
def JDToWorld(JD):
    x0 = int( JD+68570.0)    #グレゴリオ暦とユリウス暦との差を加える
    x1 = int( x0/36524.25 )  #グレゴリオ暦の 1 年 = 365.2425 年
    x2 = x0 - int(36524.25*x1 + 0.75 )
    x3 = int( ( x2+1 )/365.2425 )
    x4 = int( x2 - int( 365.25*x3 )+31.0)
    x5 = int( int(x4) / 30.59 )
    x6 = int( int(x5) / 11.0 )
    TIME=[0 for _ in range(6)]
    TIME[2] = x4 - int( 30.59*x5 )
    TIME[1] = x5 - 12*x6 + 2
    TIME[0] = 100*( x1-49 ) + x3 + x6;
    # 2月30日の補正
    if(TIME[1]==2 and TIME[2] > 28):
        if Gleap(TIME[0]):   TIME[2]=29
        else: TIME[2]=28
    tm=86400.0*(JD - int(JD))   #　1日の秒数=86,400秒
    TIME[3] = int( tm/3600.0 ); #　1時間の秒数=3,600秒
    tt3=3600.0*TIME[3]          #　1分の秒数60秒
    TIME[4] = int((tm - tt3)/60.0 )
    TIME[5] = int(tm - tt3 - 60*TIME[4] )
    return TIME
SunTab=(
    (0.0004,  31557.0, 161.0),	(0.0004,  29930.0,  48.0),
    (0.0005,   2281.0, 221.0),	(0.0005,    155.0, 118.0),#済み
    (0.0006,  33718.0, 316.0),	(0.0007,   9038.0,  64.0),
    (0.0007,   3035.0, 110.0),	(0.0007,  65929.0,  45.0),
    (0.0013,  22519.0, 352.0),	(0.0015,  45038.0, 254.0),
    (0.0018, 445267.0, 208.0),	(0.0018,     19.0, 159.0),
    (0.0020,  32964.0, 158.0),	(0.0200,  71998.1, 265.1),
    (1.9147, 35999.05,267.52))
#■度(degree)によるcos計算（数表がdegreeで与えられているため）
def cosD(deg):
    return math.cos(deg*math.pi/180)
#■角度の正規化(0度≦θ＜360度に収める)
def Nomal(TH):
    if TH >=0.0: return TH - 360.0 * int( TH / 360.0 )
    TT = -TH;TT -= 360.0 * int( TT / 360.0 )
    return 360.0 - TT
#■太陽の黄経
SunFlag=False
def SUN(t):
    th=0                            #摂動項計算
    for i in range(15):
        th += SunTab[i][0]*cosD(Nomal(SunTab[i][1]*t+SunTab[i][2]));
    th -= 0.0048 * t * cosD(Nomal(SunTab[14][1] * t + SunTab[14][2] )) 
    if SunFlag:
        return Nomal(th + Nomal(Nomal(36000.7695 * t) + 280.4659))
    return Nomal(th + Nomal(Nomal(36000.7695 * t) + 280.460692))
#■黄経(λsun)計算用時刻の設定
def ramdaSunT(tm1,tm2):
    return (tm2+0.5+tm1-2451545.0) / 36525.0
#■Δλの引き込み範囲(±180°)外のとき補正
def correctAng(dRm):
    if( dRm > 180.0 ): return dRm - 360.0
    elif( dRm < -180.0): return dRm + 360.0
    return dRm

#■時刻計算（指定された日時の直前の節気時刻）
#    tm: ユリウス日
#    A : 二分二至のとき90(度),中気のとき30(度)，
#        二十四節気のとき15(度)を指定する
#
#   【二分二至(にぶんにし)】春分・秋分・夏至・冬至のこと
#   【中気(ちゅうき)】      二十四節気のうち正節を除く12の節気。
#       雨水・春分・穀雨・小満・夏至・大暑・
#       処暑・秋分・霜降・小雪・冬至・大寒
#
def compTime(tm,A):
    tm1 = int( tm ) #時刻分解
    tm2 = tm - tm1-9.0/24.0   #JST ==> DT（時刻補正値 = 0.0sec:天保暦用）
    rmSun  = SUN(ramdaSunT(tm1, tm2)) #λsun 0 を計算
    rmSun0 = A * int(rmSun / A)       #角度区切り
    dT1   = 0       # 繰り返し計算によって直前の節気の時刻を計算する
    dT2   = 1.0     #（誤差が±1.0 sec以内になったら打ち切る。）
    DTend = 1.0 / 86400.0    #1日 = 24時間 = 1,400分 = 86,400秒
    while abs( dT1+dT2 ) > DTend :
        rmSun = SUN(ramdaSunT(tm1, tm2))   #λsun を計算
        dRm   = correctAng(rmSun - rmSun0) #黄経差 Δλ＝λsun －λsun0
        dT1 = (int)(dRm * 365.2 / 360.0)   #時刻補正値 Δt
        dT2 = dRm * 365.2 / 360.0 - dT1
        tm1 = tm1 - dT1; tm2 = tm2 - dT2 #時刻補正
        if(tm2 < 0): tm2 += 1.0; tm1 -= 1.0
    Sterm=[0, 0] #返却値領域
    Sterm[0]=tm2+9.0/24.0+tm1
    Sterm[1]=rmSun0;
    return Sterm
#■最初(直前)の二分二至の時刻計算
def SWsolstice(tm):
    return compTime(tm,90)
#■中気の時刻計算
def calcChuki(tm):
    return compTime(tm,30)
#■二十四節気の時刻計算
def calcSterm(tm):
    return compTime(tm,15)
#■二十四節気をリストとして求める
def sunTermList(Y):
    tm=WorldToJD(Y,1,1,0,0,0)
    chu=[]; chu.append(SWsolstice(tm))  
    for i in range(1,25):
        chu.append(calcSterm(chu[i-1][0]+16))
    return chu
#■二十四節気のテスト
def testSunTerm(Y,flag):
    Stab=("冬至","小寒","大寒","立春","雨水","啓蟄",
          "春分","清明","穀雨","立夏","小満","芒種",
          "夏至","小暑","大暑","立秋","処暑","白露",
          "秋分","寒露","霜降","立冬","小雪","大雪","冬至")
    chu = sunTermList(Y)
    if flag:
        print("節気,黄径,日付,時刻")
    else:
        print("\n節気 黄径  日付       時刻")
        print("---------------------------")
    for i in range(25):
        TIME=JDToWorld(chu[i][0])
        if flag:
            print(Stab[i],(",%4d, %4d/%02d/%02d, %02d:%02d:%02d" %
                           (chu[i][1],TIME[0],TIME[1],TIME[2],
                            TIME[3],TIME[4],TIME[5])))
        else:
            if i % 5 ==1:print()
            print(Stab[i],("%4d°%4d/%02d/%02d %02d:%02d" %
                           (chu[i][1],TIME[0],TIME[1],
                            TIME[2],TIME[3],TIME[4])))
    print()
#■冬至計算のテスト
def testWinter():
    for Y in range(1874,2021):
        tm=WorldToJD(Y,1,1,0,0,0)
        S=SWsolstice(tm)
        TIME=JDToWorld(S[0])
        print(Y-1, ("%4d°%4d/%02d/%02d %02d:%02d" %
               (S[1],TIME[0],TIME[1],
                TIME[2],TIME[3],TIME[4])))
               
#testSunTerm(2018,True)
#print(WorldToJDL(JtoG([100,1,1, 0,0,0])))
#testWinter()


